/* Copyright (c) 2010 Daniel Doubrovkine, All Rights Reserved
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.  
 */
package com.sun.jna.platform.win32;

import com.sun.jna.win32.StdCallLibrary;

/**
 * Ported from LMJoin.h.
 * Windows SDK 6.0A.
 * @author dblock[at]dblock.org
 */
public interface LMJoin extends StdCallLibrary {

	/**
	 * Status of a workstation.
	 */
	public abstract class NETSETUP_JOIN_STATUS {
		public static final int NetSetupUnknownStatus = 0;
		public static final int NetSetupUnjoined = 1;
		public static final int NetSetupWorkgroupName = 2;
		public static final int NetSetupDomainName = 3;		
	};

}
